﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;

namespace GoniometricFunctions
{
    public partial class Form1 : Form
    {
        private Plotter p;
        
        public Form1()
        {
            InitializeComponent();
            cmbFunc.SelectedIndex = 0;
            lstXmarks.SelectedIndices.Add(0); lstXmarks.SelectedIndices.Add(1); lstXmarks.SelectedIndices.Add(2);
            lstYmarks.SelectedIndices.Add(0); lstYmarks.SelectedIndices.Add(1); lstYmarks.SelectedIndices.Add(2);
            p = new Plotter(plot);
        }

        private void pnlColor_Click(object sender, EventArgs e)
        {
            if (colorDialog1.ShowDialog() != System.Windows.Forms.DialogResult.Cancel)
            {
                ((Panel)sender).BackColor = colorDialog1.Color;
            }
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            Clear();
        }

        private void Clear()
        {
            if (MessageBox.Show("Dojde ke smazání plochy.", "Kontrola", MessageBoxButtons.OKCancel, MessageBoxIcon.Warning) != System.Windows.Forms.DialogResult.Cancel)
            {
                p.Erase();
                ReleaseAxes();
                txtHistory.ResetText();
            }
        }

        private void btnDraw_Click(object sender, EventArgs e)
        {
            Draw();
            
        }

        private void Draw()
        {   
            if (!p.HasAxes)
            {
                AxisDefinition y = new AxisDefinition();
                AxisDefinition x = new AxisDefinition();
                if (!(GetAxisDefX(x) && GetAxisDefY(y)))
                {
                    MessageBox.Show("Neplatné parametry v nastavení os", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    return;
                }
                else
                {
                    LockAxes();
                    p.SetAxes(x, y);
                }
            }
            
            IFunction f = null;
            try
            {
                f = CreateFunction();
            }
            catch
            {
                MessageBox.Show("Neplatné parametry v nastavení funkce", "Chyba", MessageBoxButtons.OK, MessageBoxIcon.Error);
                return;
            }
            string text = String.Format("{0} * {1}({2}x) + {3}\n", f.A, cmbFunc.SelectedItem, f.B, f.C);
            txtHistory.SelectionColor = pnlColor.BackColor;
            txtHistory.AppendText(text);
            p.DrawFunction(f, pnlColor.BackColor);
        }

        private bool GetAxisDefY(AxisDefinition y)
        {
            try
            {
                y.Spread = Convert.ToUInt16(txtYspread.Text);
                
                y.ShowUnitMarks = lstYmarks.SelectedIndices.Contains(0);
                y.ShowUnitTexts = lstYmarks.SelectedIndices.Contains(2);
                y.ShowSmallMarks = lstYmarks.SelectedIndices.Contains(1);
                y.ShowSmallTexts = lstYmarks.SelectedIndices.Contains(3);
                return true;
            }
            catch
            {
                return false;
            }
            
        }

        private bool GetAxisDefX(AxisDefinition x)
        {
            try
            {
                x.Spread = Convert.ToUInt16(txtXspread.Text);
                x.ShowUnitMarks = lstXmarks.SelectedIndices.Contains(0);
                x.ShowUnitTexts = lstXmarks.SelectedIndices.Contains(2);
                x.ShowSmallMarks = lstXmarks.SelectedIndices.Contains(1);
                x.ShowSmallTexts = lstXmarks.SelectedIndices.Contains(3);
                return true;
            }
            catch
            {
                return false;
            }
        }

        private IFunction CreateFunction()
        {
            return AbstractFunction.GetFunction(cmbFunc.SelectedItem.ToString(), double.Parse(txtA.Text), double.Parse(txtB.Text), double.Parse(txtC.Text));
        }

        private void LockAxes()
        {
            txtXspread.Enabled = false;
            txtYspread.Enabled = false;
            lstXmarks.Enabled = false;
            lstYmarks.Enabled = false;
        }

        private void ReleaseAxes()
        {
            txtXspread.Enabled = true;
            txtYspread.Enabled = true;
            lstXmarks.Enabled = true;
            lstYmarks.Enabled = true;
        }

        private void vymažToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Clear();
        }

        private void vykresliToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Draw();
        }
    }
}
